# ----------------------------------------------------------------------
# Curses::UI::Language::dutch
# Maintainer: maurice@gitaar.net
# ----------------------------------------------------------------------

package Curses::UI::Language::dutch;
1;
__DATA__

# ----------------------------------------------------------------------
# For Curses::UI
# ----------------------------------------------------------------------

[screen_too_small]
Uw scherm is momenteel te klein voor deze applicatie.
Maak het scherm groter en herstart de applicatie.
Druk op <CTRL+C> om het programma te verlaten...

# ----------------------------------------------------------------------
# For Curses::UI::Calendar
# ----------------------------------------------------------------------

[months]
Januari Februari Maart April Mei Juni Juli
Augustus September Oktober November December

[days_short] Zo Ma Di Wo Do Vr Za

# ----------------------------------------------------------------------
# For Curses::UI::Buttonbox
# ----------------------------------------------------------------------

[button_ok]      o:OK
[button_cancel]  a:Annuleren
[button_yes]     j:Ja
[button_no]      n:Nee

# ----------------------------------------------------------------------
# For Curses::UI::Dialog::Error
# ----------------------------------------------------------------------

[error_title]    Foutmelding

# ----------------------------------------------------------------------
# For Curses::UI::Dialog::FileBrowser
# ----------------------------------------------------------------------

# The filebrowser title
[file_title]     Selecteer bestand
[file_savetitle] Selecteer bestand om te schrijven
[file_loadtitle] Selecteer bestand om te lezen

# The labels for the dialog screen.
[file_path]      Pad     :
[file_file]      Bestand :
[file_mask]      Masker  :

# The size of the longest label
[file_labelsize] 9

# On directory up
[file_dirup] Directory hoger

# For asking the user if a file may be overwritten
[file_overwrite_title] Bevestiging
[file_overwrite_question_pre]
Weet u zeker dat u het bestand "
[file_overwrite_question_post]
" wilt overschrijven?

# Errors
[file_err_opendir_pre] Kan directory "
[file_err_opendir_post] " niet openen
[file_err_nofileselected] U heeft nog geen bestand geselecteerd! 
